searchData={"items":[{"type":"behaviour","doc":"Allows you to paginate your Ecto results using cursors.","title":"Ouroboros","ref":"Ouroboros.html"},{"type":"behaviour","doc":"defmodule MyApp.Repo do\n      use Ecto.Repo, otp_app: :my_app\n      use Ouroboros\n    end","title":"Usage - Ouroboros","ref":"Ouroboros.html#module-usage"},{"type":"behaviour","doc":"`Ouroboros` can take any options accepted by `paginate/3`. This is useful when\nyou want to enforce some options globally across your project.","title":"Options - Ouroboros","ref":"Ouroboros.html#module-options"},{"type":"behaviour","doc":"defmodule MyApp.Repo do\n      use Ecto.Repo, otp_app: :my_app\n      use Ouroboros, limit: 10, limit_max: 100,\n    end\n\nNote that these values can be still be overriden when `paginate/3` is called.","title":"Example - Ouroboros","ref":"Ouroboros.html#module-example"},{"type":"behaviour","doc":"If you wish to avoid use of macros or you wish to use a different name for\nthe pagination function you can define your own function like so:\n\n    defmodule MyApp.Repo do\n      use Ecto.Repo, otp_app: :my_app\n\n      def my_paginate_function(query, opts \\ [], repo_opts \\ []) do\n        defaults = [limit: 10] # Default options of your choice here\n        opts = Keyword.merge(defaults, opts)\n        Ouroboros.paginate(query, opts, __MODULE__, repo_opts)\n      end\n    end","title":"Use without macros - Ouroboros","ref":"Ouroboros.html#module-use-without-macros"},{"type":"function","doc":"Generate a cursor for the supplied record, in the same manner as the\n`before` and `after` cursors generated by `paginate/3`.\n\nFor the cursor to be compatible with `paginate/3`, `fields`\nmust have the same value as the `fields` option passed to it.","title":"Ouroboros.cursor_for_record/3","ref":"Ouroboros.html#cursor_for_record/3"},{"type":"function","doc":"iex> Ouroboros.cursor_for_record(%Ouroboros.Customer{id: 1}, [:id])\n    \"g2sAAQE\"\n\n    iex> Ouroboros.cursor_for_record(%Ouroboros.Customer{id: 1, name: \"Alice\"}, [id: :asc, name: :desc])\n    \"g2wAAAACYQFtAAAABUFsaWNlag\"","title":"Example - Ouroboros.cursor_for_record/3","ref":"Ouroboros.html#cursor_for_record/3-example"},{"type":"callback","doc":"Fetches all the results matching the query within the cursors.","title":"Ouroboros.paginate/3","ref":"Ouroboros.html#c:paginate/3"},{"type":"callback","doc":"* `:after` - fetch the records after this cursor;\n  * `:before` - fetch the records before this cursor;\n  * `:fields` - fields with sorting direction used to determine the cursor.\n    In most cases, this should be the same fields as the ones used for sorting in the query.\n    When you use named bindings in your query they can also be provided;\n  * `:value_fun` - function of arity 2 to lookup cursor values on returned records.\n    Defaults to `&Ouroboros.value_fun_default/2`;\n  * `:limit` - limits the number of records returned per page. Note that this\n    number will be capped by `:limit_max`. Defaults to `50`;\n  * `:limit_max` - sets a maximum cap for `:limit`. This option can be useful when `:limit`\n    is set dynamically (e.g from a URL param set by a user) but you still want to\n    enfore a maximum. Defaults to `100`.","title":"Options - Ouroboros.paginate/3","ref":"Ouroboros.html#c:paginate/3-options"},{"type":"callback","doc":"This will be passed directly to `Ecto.Repo.all/2`, as such any option supported\nby this function can be used here.","title":"Repo options - Ouroboros.paginate/3","ref":"Ouroboros.html#c:paginate/3-repo-options"},{"type":"callback","doc":"query = from(p in Post, order_by: [asc: p.inserted_at, asc: p.id], select: p)\n\n    Repo.paginate(query, fields: [:inserted_at, :id], limit: 50)","title":"Simple example - Ouroboros.paginate/3","ref":"Ouroboros.html#c:paginate/3-simple-example"},{"type":"callback","doc":"query = from(p in Post, order_by: [asc: p.inserted_at, desc: p.id], select: p)\n\n    Repo.paginate(query, fields: [inserted_at: :asc, id: :desc], limit: 50)","title":"Example with using custom sort directions per field - Ouroboros.paginate/3","ref":"Ouroboros.html#c:paginate/3-example-with-using-custom-sort-directions-per-field"},{"type":"callback","doc":"from p in Post, as: :posts,\n      join: a in assoc(p, :author), as: :author,\n      preload: [author: a],\n      select: p,\n      order_by: [asc: a.name, asc: p.id]\n\n    Repo.paginate(query, fields: [{{:author, :name}, :asc}, id: :asc], limit: 50)\n\nWhen sorting on columns in joined tables it is necessary to use named bindings. In\nthis case we name it `author`. In the `fields` we refer to this named binding\nand its column name.\n\nTo build the cursor Ouroboros uses the returned Ecto.Schema. When using a joined\ncolumn the returned Ecto.Schema won't have the value of the joined column\nunless we preload it. E.g. in this case the cursor will be build up from\n`post.id` and `post.author.name`. This presupposes that the named of the\nbinding is the same as the name of the relationship on the original struct.\n\nOne level deep joins are supported out of the box but if we join on a second\nlevel, e.g. `post.author.company.name` a custom function can be supplied to\nhandle the cursor value retrieval. This also applies when the named binding\ndoes not map to the name of the relationship.","title":"Example with sorting on columns in joined tables - Ouroboros.paginate/3","ref":"Ouroboros.html#c:paginate/3-example-with-sorting-on-columns-in-joined-tables"},{"type":"callback","doc":"from p in Post, as: :posts,\n      join: a in assoc(p, :author), as: :author,\n      join: c in assoc(a, :company), as: :company,\n      preload: [author: a],\n      select: p,\n      order_by: [\n        {:asc, a.name},\n        {:asc, p.id}\n      ]\n\n    Repo.paginate(query,\n      fields: [{{:company, :name}, :asc}, id: :asc],\n      value_fun: fn\n        post, {{:company, name}, _} -> {:string, post.author.company.name}\n        post, field -> Ouroboros.value_fun_default(post, field)\n      end, limit: 50)","title":"Example - Ouroboros.paginate/3","ref":"Ouroboros.html#c:paginate/3-example"},{"type":"function","doc":"Default function used to get the value of a cursor field from the supplied\nmap. This function can be overriden in the `Ouroboros.Config` using the `value_fun` key.\n\nWhen using named bindings to sort on joined columns it will attempt to get\nthe value of joined column by using the named binding as the name of the\nrelationship on the original Ecto.Schema.","title":"Ouroboros.type_fun_default/2","ref":"Ouroboros.html#type_fun_default/2"},{"type":"function","doc":"iex> Ouroboros.type_fun_default(%Ouroboros.Customer{id: 1}, :id)\n    :id\n\n    iex> Ouroboros.type_fun_default(%Ouroboros.Customer{}, {:address, :city})\n    :string\n\n    iex> Ouroboros.type_fun_default(%Ouroboros.Customer{id: 1, address: %Ouroboros.Address{city: \"London\"}}, {:address, :city})\n    :string\n\n    iex> Ouroboros.type_fun_default(Ouroboros.Customer, :inserted_at)\n    :naive_datetime","title":"Example - Ouroboros.type_fun_default/2","ref":"Ouroboros.html#type_fun_default/2-example"},{"type":"function","doc":"Default function used to get the value of a cursor field from the supplied\nmap. This function can be overriden in the `Ouroboros.Config` using the `value_fun` key.\n\nWhen using named bindings to sort on joined columns it will attempt to get\nthe value of joined column by using the named binding as the name of the\nrelationship on the original Ecto.Schema.","title":"Ouroboros.value_fun_default/2","ref":"Ouroboros.html#value_fun_default/2"},{"type":"function","doc":"iex> Ouroboros.value_fun_default(%Ouroboros.Customer{id: 1}, :id)\n    {:id, 1}\n\n    iex> Ouroboros.value_fun_default(%Ouroboros.Customer{id: 1, address: %Ouroboros.Address{city: \"London\"}}, {:address, :city})\n    {:string, \"London\"}","title":"Example - Ouroboros.value_fun_default/2","ref":"Ouroboros.html#value_fun_default/2-example"},{"type":"module","doc":"","title":"Ouroboros.Address","ref":"Ouroboros.Address.html"},{"type":"module","doc":"","title":"Ouroboros.Case","ref":"Ouroboros.Case.html"},{"type":"module","doc":"","title":"Ouroboros.Customer","ref":"Ouroboros.Customer.html"},{"type":"function","doc":"","title":"Ouroboros.Customer.active/1","ref":"Ouroboros.Customer.html#active/1"},{"type":"module","doc":"","title":"Ouroboros.Factory","ref":"Ouroboros.Factory.html"},{"type":"function","doc":"","title":"Ouroboros.Factory.address_factory/0","ref":"Ouroboros.Factory.html#address_factory/0"},{"type":"function","doc":"","title":"Ouroboros.Factory.build/2","ref":"Ouroboros.Factory.html#build/2"},{"type":"function","doc":"","title":"Ouroboros.Factory.build_list/3","ref":"Ouroboros.Factory.html#build_list/3"},{"type":"function","doc":"","title":"Ouroboros.Factory.build_pair/2","ref":"Ouroboros.Factory.html#build_pair/2"},{"type":"function","doc":"","title":"Ouroboros.Factory.create/1","ref":"Ouroboros.Factory.html#create/1"},{"type":"function","doc":"","title":"Ouroboros.Factory.create/2","ref":"Ouroboros.Factory.html#create/2"},{"type":"function","doc":"","title":"Ouroboros.Factory.create_list/3","ref":"Ouroboros.Factory.html#create_list/3"},{"type":"function","doc":"","title":"Ouroboros.Factory.create_pair/2","ref":"Ouroboros.Factory.html#create_pair/2"},{"type":"function","doc":"","title":"Ouroboros.Factory.customer_factory/0","ref":"Ouroboros.Factory.html#customer_factory/0"},{"type":"function","doc":"Raises a helpful error if no factory is defined.","title":"Ouroboros.Factory.factory/1","ref":"Ouroboros.Factory.html#factory/1"},{"type":"function","doc":"","title":"Ouroboros.Factory.insert/1","ref":"Ouroboros.Factory.html#insert/1"},{"type":"function","doc":"","title":"Ouroboros.Factory.insert/2","ref":"Ouroboros.Factory.html#insert/2"},{"type":"function","doc":"","title":"Ouroboros.Factory.insert/3","ref":"Ouroboros.Factory.html#insert/3"},{"type":"function","doc":"","title":"Ouroboros.Factory.insert_list/3","ref":"Ouroboros.Factory.html#insert_list/3"},{"type":"function","doc":"","title":"Ouroboros.Factory.insert_list/4","ref":"Ouroboros.Factory.html#insert_list/4"},{"type":"function","doc":"","title":"Ouroboros.Factory.insert_pair/2","ref":"Ouroboros.Factory.html#insert_pair/2"},{"type":"function","doc":"","title":"Ouroboros.Factory.insert_pair/3","ref":"Ouroboros.Factory.html#insert_pair/3"},{"type":"function","doc":"","title":"Ouroboros.Factory.params_for/2","ref":"Ouroboros.Factory.html#params_for/2"},{"type":"function","doc":"","title":"Ouroboros.Factory.params_with_assocs/2","ref":"Ouroboros.Factory.html#params_with_assocs/2"},{"type":"function","doc":"","title":"Ouroboros.Factory.payment_factory/0","ref":"Ouroboros.Factory.html#payment_factory/0"},{"type":"function","doc":"","title":"Ouroboros.Factory.string_params_for/2","ref":"Ouroboros.Factory.html#string_params_for/2"},{"type":"function","doc":"","title":"Ouroboros.Factory.string_params_with_assocs/2","ref":"Ouroboros.Factory.html#string_params_with_assocs/2"},{"type":"module","doc":"","title":"Ouroboros.Migration","ref":"Ouroboros.Migration.html"},{"type":"function","doc":"","title":"Ouroboros.Migration.change/0","ref":"Ouroboros.Migration.html#change/0"},{"type":"module","doc":"Defines a page.","title":"Ouroboros.Page","ref":"Ouroboros.Page.html"},{"type":"module","doc":"* `entries` - a list entries contained in this page.\n* `metadata` - metadata attached to this page.","title":"Fields - Ouroboros.Page","ref":"Ouroboros.Page.html#module-fields"},{"type":"type","doc":"","title":"Ouroboros.Page.t/0","ref":"Ouroboros.Page.html#t:t/0"},{"type":"module","doc":"Defines page metadata.","title":"Ouroboros.Page.Metadata","ref":"Ouroboros.Page.Metadata.html"},{"type":"module","doc":"* `after` - cursor representing the last row of the current page;\n* `before` - cursor representing the first row of the current page;\n* `total` - total number of entries;\n* `limit` - the maximum number of entries that can be contained in this page.","title":"Fields - Ouroboros.Page.Metadata","ref":"Ouroboros.Page.Metadata.html#module-fields"},{"type":"type","doc":"","title":"Ouroboros.Page.Metadata.t/0","ref":"Ouroboros.Page.Metadata.html#t:t/0"},{"type":"module","doc":"","title":"Ouroboros.Payment","ref":"Ouroboros.Payment.html"},{"type":"function","doc":"","title":"Ouroboros.Payment.failed/1","ref":"Ouroboros.Payment.html#failed/1"},{"type":"function","doc":"","title":"Ouroboros.Payment.successful/1","ref":"Ouroboros.Payment.html#successful/1"},{"type":"module","doc":"","title":"Ouroboros.Repo","ref":"Ouroboros.Repo.html"},{"type":"function","doc":"","title":"Ouroboros.Repo.aggregate/3","ref":"Ouroboros.Repo.html#aggregate/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.aggregate/4","ref":"Ouroboros.Repo.html#aggregate/4"},{"type":"function","doc":"","title":"Ouroboros.Repo.all/2","ref":"Ouroboros.Repo.html#all/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.checked_out?/0","ref":"Ouroboros.Repo.html#checked_out?/0"},{"type":"function","doc":"","title":"Ouroboros.Repo.checkout/2","ref":"Ouroboros.Repo.html#checkout/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.child_spec/1","ref":"Ouroboros.Repo.html#child_spec/1"},{"type":"function","doc":"","title":"Ouroboros.Repo.config/0","ref":"Ouroboros.Repo.html#config/0"},{"type":"function","doc":"","title":"Ouroboros.Repo.default_options/1","ref":"Ouroboros.Repo.html#default_options/1"},{"type":"function","doc":"","title":"Ouroboros.Repo.delete/2","ref":"Ouroboros.Repo.html#delete/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.delete!/2","ref":"Ouroboros.Repo.html#delete!/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.delete_all/2","ref":"Ouroboros.Repo.html#delete_all/2"},{"type":"function","doc":"A convenience function for SQL-based repositories that forces all connections in the\npool to disconnect within the given interval.\n\nSee `Ecto.Adapters.SQL.disconnect_all/3` for more information.","title":"Ouroboros.Repo.disconnect_all/2","ref":"Ouroboros.Repo.html#disconnect_all/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.exists?/2","ref":"Ouroboros.Repo.html#exists?/2"},{"type":"function","doc":"A convenience function for SQL-based repositories that executes an EXPLAIN statement or similar\ndepending on the adapter to obtain statistics for the given query.\n\nSee `Ecto.Adapters.SQL.explain/4` for more information.","title":"Ouroboros.Repo.explain/3","ref":"Ouroboros.Repo.html#explain/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.get/3","ref":"Ouroboros.Repo.html#get/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.get!/3","ref":"Ouroboros.Repo.html#get!/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.get_by/3","ref":"Ouroboros.Repo.html#get_by/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.get_by!/3","ref":"Ouroboros.Repo.html#get_by!/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.get_dynamic_repo/0","ref":"Ouroboros.Repo.html#get_dynamic_repo/0"},{"type":"function","doc":"","title":"Ouroboros.Repo.in_transaction?/0","ref":"Ouroboros.Repo.html#in_transaction?/0"},{"type":"function","doc":"","title":"Ouroboros.Repo.init/2","ref":"Ouroboros.Repo.html#init/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.insert/2","ref":"Ouroboros.Repo.html#insert/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.insert!/2","ref":"Ouroboros.Repo.html#insert!/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.insert_all/3","ref":"Ouroboros.Repo.html#insert_all/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.insert_or_update/2","ref":"Ouroboros.Repo.html#insert_or_update/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.insert_or_update!/2","ref":"Ouroboros.Repo.html#insert_or_update!/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.load/2","ref":"Ouroboros.Repo.html#load/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.one/2","ref":"Ouroboros.Repo.html#one/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.one!/2","ref":"Ouroboros.Repo.html#one!/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.paginate/3","ref":"Ouroboros.Repo.html#paginate/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.preload/3","ref":"Ouroboros.Repo.html#preload/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.prepare_query/3","ref":"Ouroboros.Repo.html#prepare_query/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.put_dynamic_repo/1","ref":"Ouroboros.Repo.html#put_dynamic_repo/1"},{"type":"function","doc":"A convenience function for SQL-based repositories that executes the given query.\n\nSee `Ecto.Adapters.SQL.query/4` for more information.","title":"Ouroboros.Repo.query/3","ref":"Ouroboros.Repo.html#query/3"},{"type":"function","doc":"A convenience function for SQL-based repositories that executes the given query.\n\nSee `Ecto.Adapters.SQL.query!/4` for more information.","title":"Ouroboros.Repo.query!/3","ref":"Ouroboros.Repo.html#query!/3"},{"type":"function","doc":"A convenience function for SQL-based repositories that executes the given multi-result query.\n\nSee `Ecto.Adapters.SQL.query_many/4` for more information.","title":"Ouroboros.Repo.query_many/3","ref":"Ouroboros.Repo.html#query_many/3"},{"type":"function","doc":"A convenience function for SQL-based repositories that executes the given multi-result query.\n\nSee `Ecto.Adapters.SQL.query_many!/4` for more information.","title":"Ouroboros.Repo.query_many!/3","ref":"Ouroboros.Repo.html#query_many!/3"},{"type":"function","doc":"","title":"Ouroboros.Repo.reload/2","ref":"Ouroboros.Repo.html#reload/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.reload!/2","ref":"Ouroboros.Repo.html#reload!/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.rollback/1","ref":"Ouroboros.Repo.html#rollback/1"},{"type":"function","doc":"","title":"Ouroboros.Repo.start_link/1","ref":"Ouroboros.Repo.html#start_link/1"},{"type":"function","doc":"","title":"Ouroboros.Repo.stop/1","ref":"Ouroboros.Repo.html#stop/1"},{"type":"function","doc":"","title":"Ouroboros.Repo.stream/2","ref":"Ouroboros.Repo.html#stream/2"},{"type":"function","doc":"A convenience function for SQL-based repositories that translates the given query to SQL.\n\nSee `Ecto.Adapters.SQL.to_sql/3` for more information.","title":"Ouroboros.Repo.to_sql/2","ref":"Ouroboros.Repo.html#to_sql/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.transaction/2","ref":"Ouroboros.Repo.html#transaction/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.update/2","ref":"Ouroboros.Repo.html#update/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.update!/2","ref":"Ouroboros.Repo.html#update!/2"},{"type":"function","doc":"","title":"Ouroboros.Repo.update_all/3","ref":"Ouroboros.Repo.html#update_all/3"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}